@@ -22,6 +22,8 @@
 
 #include <config.h>
 
+#include <stdlib.h>
+
 #include "gnome-cups-ui-driver.h"
 #include "gnome-cups-ui-util.h"
 #include <libgnomecups/gnome-cups-request.h>
@@ -40,6 +42,7 @@
 #include <gtk/gtkfilechooser.h>
 #include <gtk/gtkfilechooserdialog.h>
 #include <gtk/gtkmessagedialog.h>
+#include <gtk/gtklabel.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 #include <glib/gstrfuncs.h>
@@ -286,7 +289,7 @@ ppd_extract_model_and_driver (GCupsPPD *ppd)
 		add_vendor_aliases ("HP",	hp);
 		add_vendor_aliases ("Dymo",	dymo);
 		add_vendor_aliases ("Canon",	canon);
-		add_vendor_aliases ("Generic",	generic);
+		add_vendor_aliases (_("Generic"), generic);
 	}
 
 	/* clean up the vendor */
@@ -619,6 +622,15 @@ select_model (GladeXML *xml, char const *model_name)
 	} while (gtk_tree_model_iter_next (model, &iter));
 }
 
+static void
+set_make_and_model (GladeXML *xml, const char *make_and_model)
+{
+	GtkWidget *label;
+	
+	label = glade_xml_get_widget (xml, "make_and_model_label");
+	gtk_label_set_text (GTK_LABEL (label), make_and_model);
+}
+
 static char *
 get_selected_vendor (GladeXML *xml)
 {
@@ -835,13 +847,34 @@ cb_install_driver (GCupsDriverSelector *ds)
 		err = g_error_new (1, 1, _("Only files ending with .ppd or .ppd.gz will be installed"));
 		goto done;
 	}
-
+	
 	if (!g_file_test (CUPS_DIR, G_FILE_TEST_IS_DIR)) {
 		err = g_error_new (1, 1,
 			_("CUPS is installed differently than expected.  There is no directory '%s'"), CUPS_DIR);
 		goto done;
 	}
-
+	
+	if (length > 7 && !g_ascii_strcasecmp (ppd_file+length-7, ".ppd.gz")) {
+		char *command, *name;
+		const char *gzip;
+		
+		if (!(gzip = g_find_program_in_path ("gzip")))
+			goto done;
+		
+		name = g_path_get_basename (ppd_file);
+		name[strlen (name) - 3] = '\0';
+		
+		command = g_strdup_printf ("%s -cd %s > %s/%s", gzip, ppd_file, g_get_tmp_dir (), name);
+		system (command);
+		g_free (command);
+		
+		ppd_file = g_strdup_printf ("%s/%s", g_get_tmp_dir (), name);
+		g_free (name);
+		
+		if (!g_file_test (ppd_file, G_FILE_TEST_EXISTS))
+			goto done;
+	}
+	
 	base = g_path_get_basename  (ppd_file);
 	target_path = g_build_filename (CUPS_DIR, base, NULL);
 
@@ -1038,7 +1071,9 @@ gcups_driver_selector_set_nickname (GCupsDriverSelector *ds, const char *nicknam
 	GCupsPPD *ppd;
 
 	g_return_if_fail (ds != NULL);
-
+	
+	set_make_and_model (ds->xml, nickname);
+	
 	ppd = get_detected_ppd (ds, nickname);
 	if (ppd != NULL) {
 		combo_select (ds->xml, "vendors", 0, ppd->vendor);
